fragment reviewThreadData on PullRequestReviewThread {
  id
  isResolved
  comments(first: 100) {
    totalCount
    nodes {
      pullRequestReview {
        id
      }
      ...commentData    # fragment: comment_frag.graphql
    }
  }
}
